@@ -1,42 +1,34 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from __future__ import unicode_literals
-import copy
-from future.utils import python_2_unicode_compatible, string_types, text_type
 import uuid
+from typing import Optional, Any, Tuple
 
 
-@python_2_unicode_compatible
 class Node(object):
     def __init__(
         self,
-        identifier=None,
-        auto_uuid=True,
-        keyed=True,
-        accept_children=True,
-        repr_=None,
-        data=None,
-    ):
+        identifier: Optional[str] = None,
+        auto_uuid: bool = True,
+        keyed: bool = True,
+        accept_children: bool = True,
+        repr_: Optional[str] = None,
+        data: Any = None,
+    ) -> None:
         """
         :param identifier: node identifier, must be unique per tree
         """
-        if identifier is not None and not isinstance(identifier, string_types):
-            raise ValueError(
-                "Identifier must be a string type, provided type is <%s>"
-                % type(identifier)
-            )
         if identifier is None:
             if not auto_uuid:
                 raise ValueError("Required identifier")
-            identifier = uuid.uuid4()
+            identifier = str(uuid.uuid4())
         self.identifier = identifier
         self.keyed = keyed
         self.accept_children = accept_children
         self.repr = repr_
         self.data = data
 
-    def line_repr(self, depth, **kwargs):
+    def line_repr(self, depth: int, **kwargs) -> Tuple[str, str]:
         """Control how node is displayed in tree representation.
         _
         ├── one                                           end
@@ -46,18 +38,18 @@ class Node(object):
         if self.repr is not None:
             return self.repr, ""
         if not self.accept_children:
-            return text_type(self.data), ""
+            return str(self.data), ""
         if self.keyed:
             return "{}", ""
         return "[]", ""
 
-    def __eq__(self, other):
+    def __eq__(self, other) -> bool:
         if not isinstance(other, self.__class__):
             return False
         return self.identifier == other.identifier
 
-    def __str__(self):
+    def __str__(self) -> str:
         return "%s, id=%s" % (self.__class__.__name__, self.identifier)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return self.__str__()
